@@ -105,7 +105,7 @@ Fields for use with PostGIS types.
 use strict;
 use warnings;
 our ( $DEBUG, $WARN );
-our $VERSION = '1.63';
+our $VERSION = '1.64';
 $DEBUG = 0 unless defined $DEBUG;
 
 use base qw(SQL::Translator::Producer);
@@ -204,6 +204,7 @@ and table_constraint is:
   { UNIQUE ( column_name [, ... ] ) |
     PRIMARY KEY ( column_name [, ... ] ) |
     CHECK ( expression ) |
+    EXCLUDE [USING acc_method] (expression) [INCLUDE (column [, ...])] [WHERE (predicate)]
     FOREIGN KEY ( column_name [, ... ] ) REFERENCES reftable [ ( refcolumn [, ... ] ) ]
       [ MATCH FULL | MATCH PARTIAL ] [ ON DELETE action ] [ ON UPDATE action ] }
   [ DEFERRABLE | NOT DEFERRABLE ] [ INITIALLY DEFERRED | INITIALLY IMMEDIATE ]
@@ -493,6 +494,20 @@ sub create_view {
     return $create;
 }
 
+# Returns a enum custom type name and list of values iff the field looks like an enum.
+sub _enum_typename_and_values {
+    my $field = shift;
+    if (ref $field->extra->{list} eq 'ARRAY') { # can't do anything unless we know the list
+        if ($field->extra->{custom_type_name}) {
+            return ( $field->extra->{custom_type_name}, $field->extra->{list} );
+        } elsif ($field->data_type eq 'enum') {
+            my $name= $field->table->name . '_' . $field->name . '_type';
+            return ( $name, $field->extra->{list} );
+        }
+    }
+    return ();
+}
+
 {
 
     my %field_name_scope;
@@ -524,18 +539,17 @@ sub create_view {
         #
         my $data_type = lc $field->data_type;
         my %extra     = $field->extra;
-        my $list      = $extra{'list'} || [];
-        my $commalist = join( ', ', map { __PACKAGE__->_quote_string($_) } @$list );
+        my ($enum_typename, $list) = _enum_typename_and_values($field);
 
-        if ($postgres_version >= 8.003 && $data_type eq 'enum') {
-            my $type_name = $extra{'custom_type_name'} || $field->table->name . '_' . $field->name . '_type';
-            $field_def .= ' '. $type_name;
-            my $new_type_def = "DROP TYPE IF EXISTS $type_name CASCADE;\n" .
-                               "CREATE TYPE $type_name AS ENUM ($commalist)";
-            if (! exists $type_defs->{$type_name} ) {
-                $type_defs->{$type_name} = $new_type_def;
-            } elsif ( $type_defs->{$type_name} ne $new_type_def ) {
-                die "Attempted to redefine type name '$type_name' as a different type.\n";
+        if ($postgres_version >= 8.003 && $enum_typename) {
+            my $commalist = join( ', ', map { __PACKAGE__->_quote_string($_) } @$list );
+            $field_def .= ' '. $enum_typename;
+            my $new_type_def = "DROP TYPE IF EXISTS $enum_typename CASCADE;\n" .
+                               "CREATE TYPE $enum_typename AS ENUM ($commalist)";
+            if (! exists $type_defs->{$enum_typename} ) {
+                $type_defs->{$enum_typename} = $new_type_def;
+            } elsif ( $type_defs->{$enum_typename} ne $new_type_def ) {
+                die "Attempted to redefine type name '$enum_typename' as a different type.\n";
             }
         } else {
             $field_def .= ' '. convert_datatype($field);
@@ -602,6 +616,19 @@ sub create_geometry_constraints {
     return @constraints;
 }
 
+sub _extract_extras_from_options {
+  my ($options_haver, $dispatcher) = @_;
+  for my $opt ($options_haver->options) {
+    if (ref $opt eq 'HASH') {
+      for my $key (keys %$opt) {
+        my $val = $opt->{$key};
+        next unless defined $val;
+        $dispatcher->{lc $key}->($val);
+      }
+    }
+  }
+}
+
 {
     my %index_name;
     sub create_index
@@ -623,26 +650,17 @@ sub create_geometry_constraints {
         return unless @fields;
 
         my %index_extras;
-        for my $opt ( $index->options ) {
-            if ( ref $opt eq 'HASH' ) {
-                foreach my $key (keys %$opt) {
-                    my $value = $opt->{$key};
-                    next unless defined $value;
-                    if ( uc($key) eq 'USING' ) {
-                        $index_extras{using} = "USING $value";
-                    }
-                    elsif ( uc($key) eq 'WHERE' ) {
-                        $index_extras{where} = "WHERE $value";
-                    }
-                    elsif ( uc($key) eq 'INCLUDE' ) {
-                        next unless $postgres_version >= 11;
-                        die 'Include list must be an arrayref' unless ref $value eq 'ARRAY';
-                        my $value_list = join ', ', @$value;
-                        $index_extras{include} = "INCLUDE ($value_list)"
-                    }
-                }
+        _extract_extras_from_options($index, {
+            using => sub { $index_extras{using} = "USING $_[0]" },
+            where => sub { $index_extras{where} = "WHERE $_[0]" },
+            include => sub {
+                my ($value) = @_;
+                return unless $postgres_version >= 11;
+                die 'Include list must be an arrayref' unless ref $value eq 'ARRAY';
+                my $value_list = join ', ', @$value;
+                $index_extras{include} = "INCLUDE ($value_list)"
             }
-        }
+        });
 
         my $def_start = 'CONSTRAINT ' . $generator->quote($name) . ' ';
         my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ( $generator->quote($_) ) } @fields)) . ')';
@@ -671,8 +689,21 @@ sub create_constraint
     my ($c, $options) = @_;
 
     my $generator = _generator($options);
+    my $postgres_version = $options->{postgres_version} || 0;
     my $table_name = $c->table->name;
     my (@constraint_defs, @fks);
+    my %constraint_extras;
+    _extract_extras_from_options($c, {
+        using => sub { $constraint_extras{using} = "USING $_[0]" },
+        where => sub { $constraint_extras{where} = "WHERE ( $_[0] )" },
+        include => sub {
+          my ($value) = @_;
+          return unless $postgres_version >= 11;
+          die 'Include list must be an arrayref' unless ref $value eq 'ARRAY';
+          my $value_list = join ', ', @$value;
+          $constraint_extras{include} = "INCLUDE ( $value_list )"
+        },
+      });
 
     my $name = $c->name || '';
 
@@ -680,22 +711,23 @@ sub create_constraint
 
     my @rfields = grep { defined } $c->reference_fields;
 
-    next if !@fields && $c->type ne CHECK_C;
-    my $def_start = $name ? 'CONSTRAINT ' . $generator->quote($name) . ' ' : '';
+    return if !@fields && ($c->type ne CHECK_C && $c->type ne EXCLUDE);
+    my $def_start = $name ? 'CONSTRAINT ' . $generator->quote($name) : '';
     my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ( $generator->quote($_) ) } @fields)) . ')';
+    my $include      =  $constraint_extras{include} || '';
     if ( $c->type eq PRIMARY_KEY ) {
-        push @constraint_defs, "${def_start}PRIMARY KEY ".$field_names;
+        push @constraint_defs, join ' ', grep $_, $def_start, "PRIMARY KEY", $field_names, $include;
     }
     elsif ( $c->type eq UNIQUE ) {
-        push @constraint_defs, "${def_start}UNIQUE " .$field_names;
+        push @constraint_defs, join ' ', grep $_, $def_start, "UNIQUE", $field_names, $include;
     }
     elsif ( $c->type eq CHECK_C ) {
         my $expression = $c->expression;
-        push @constraint_defs, "${def_start}CHECK ($expression)";
+        push @constraint_defs, join ' ', grep $_, $def_start, "CHECK ($expression)";
     }
     elsif ( $c->type eq FOREIGN_KEY ) {
-        my $def .= "ALTER TABLE " . $generator->quote($table_name) . " ADD ${def_start}FOREIGN KEY $field_names"
-            . "\n  REFERENCES " . $generator->quote($c->reference_table);
+        my $def .= join ' ', grep $_, "ALTER TABLE", $generator->quote($table_name), 'ADD', $def_start, "FOREIGN KEY $field_names";
+        $def .= "\n  REFERENCES " . $generator->quote($c->reference_table);
 
         if ( @rfields ) {
             $def .= ' (' . join( ', ', map { $generator->quote($_) } @rfields ) . ')';
@@ -720,6 +752,12 @@ sub create_constraint
 
         push @fks, "$def";
     }
+    elsif( $c->type eq EXCLUDE ) {
+        my $using = $constraint_extras{using} || '';
+        my $expression =  $c->expression;
+        my $where      =  $constraint_extras{where} || '';
+        push @constraint_defs, join ' ', grep $_, $def_start, 'EXCLUDE', $using, "( $expression )", $include, $where;
+    }
 
     return \@constraint_defs, \@fks;
 }
@@ -896,6 +934,25 @@ sub alter_field
                    )
         if($to_dt ne $from_dt);
 
+    my ($from_enum_typename, $from_list) = _enum_typename_and_values($from_field);
+    my ($to_enum_typename,   $to_list  ) = _enum_typename_and_values($to_field);
+    if ($from_enum_typename && $to_enum_typename && $from_enum_typename eq $to_enum_typename) {
+        # See if new enum values were added, and update the enum
+        my %existing_vals = map +($_ => 1), @$from_list;
+        my %desired_vals = map +($_ => 1), @$to_list;
+        my @add_vals = grep !$existing_vals{$_}, keys %desired_vals;
+        my @del_vals = grep !$desired_vals{$_}, keys %existing_vals;
+        my $pg_ver_ok= ($options->{postgres_version} || 0) >= 9.001;
+        push @out, '-- Set $sqlt->producer_args->{postgres_version} >= 9.001 to alter enums'
+            if !$pg_ver_ok && @add_vals;
+        for (@add_vals) {
+            push @out, sprintf '%sALTER TYPE %s ADD VALUE IF NOT EXISTS %s',
+                ($pg_ver_ok? '':'-- '), $to_enum_typename, $generator->quote_string($_);
+        }
+        push @out, "-- Unimplemented: delete values from enum type '$to_enum_typename': ".join(", ", @del_vals)
+            if @del_vals;
+    }
+
     my $old_default = $from_field->default_value;
     my $new_default = $to_field->default_value;
     my $default_value = $to_field->default_value;
