@@ -35,6 +35,8 @@ This module exports the following constants for Schema features;
 
 =item UNIQUE
 
+=item EXCLUDE
+
 =back
 
 =cut
@@ -43,7 +45,7 @@ use strict;
 use warnings;
 use base qw( Exporter );
 require Exporter;
-our $VERSION = '1.63';
+our $VERSION = '1.64';
 
 our @EXPORT = qw[
     CHECK_C
@@ -55,6 +57,7 @@ our @EXPORT = qw[
     NULL
     PRIMARY_KEY
     UNIQUE
+    EXCLUDE
 ];
 
 #
@@ -78,6 +81,8 @@ use constant PRIMARY_KEY => 'PRIMARY KEY';
 
 use constant UNIQUE => 'UNIQUE';
 
+use constant EXCLUDE => 'EXCLUDE';
+
 1;
 
 =pod
