@@ -7,8 +7,9 @@ use File::Spec;
 use Scalar::Util qw(blessed);
 use Try::Tiny;
 use Carp qw(carp croak);
+use List::Util qw(any);
 
-our $VERSION = '1.63';
+our $VERSION = '1.64';
 
 use base qw(Exporter);
 our @EXPORT_OK = qw(
@@ -131,7 +132,7 @@ sub parse_list_arg {
     #
     # This protects stringification of references.
     #
-    if ( @$list && ref $list->[0] ) {
+    if (any { ref $_ } @$list ) {
         return $list;
     }
     #
